@@ -14,6 +14,7 @@ NOTE: parameters are ordered with precedence of (time, worker type, firm type).
 '''
 import numpy as np
 from qpsolvers import solve_qp
+from scipy.sparse import csc_matrix
 from bipartitepandas.util import to_list
 
 class QPConstrained:
@@ -162,7 +163,7 @@ class QPConstrained:
 
         return cons.res is not None
 
-    def solve(self, P, q):
+    def solve(self, P, q, solver='quadprog', verbose=False, **kwargs):
         '''
         Solve a quadratic programming model of the following form:
             min_x(1/2 x.T @ P @ x + q.T @ x)
@@ -172,18 +173,28 @@ class QPConstrained:
         Arguments:
             P (NumPy Array): P in quadratic programming problem
             q (NumPy Array): q in quadratic programming problem
+            solver (str): solver to use
+            verbose (bool): if True, print extra output
+            **kwargs: parameters for solver
 
         Returns:
             (NumPy Array): x that solves quadratic programming problem
         '''
-        if len(self.G) > 0 and len(self.A) > 0:
-            self.res = solve_qp(P=P, q=q, G=self.G, h=self.h, A=self.A, b=self.b)
-        elif len(self.G) > 0:
-            self.res = solve_qp(P=P, q=q, G=self.G, h=self.h)
-        elif len(self.A) > 0:
-            self.res = solve_qp(P=P, q=q, A=self.A, b=self.b)
+        if solver in ['ecos', 'gurobi', 'mosek', 'osqp', 'qpswift', 'scs']:
+            # If using sparse solver
+            if (self.G.shape[0] > 0) and not isinstance(self.G, csc_matrix):
+                self.G = csc_matrix(self.G)
+            if (self.A.shape[0] > 0) and not isinstance(self.A, csc_matrix):
+                self.A = csc_matrix(self.A)
+
+        if self.G.shape[0] > 0 and self.A.shape[0] > 0:
+            self.res = solve_qp(P=P, q=q, G=self.G, h=self.h, A=self.A, b=self.b, solver=solver, verbose=verbose, **kwargs)
+        elif self.G.shape[0] > 0:
+            self.res = solve_qp(P=P, q=q, G=self.G, h=self.h, solver=solver, verbose=verbose, **kwargs)
+        elif self.A.shape[0] > 0:
+            self.res = solve_qp(P=P, q=q, A=self.A, b=self.b, solver=solver, verbose=verbose, **kwargs)
         else:
-            self.res = solve_qp(P=P, q=q)
+            self.res = solve_qp(P=P, q=q, solver=solver, verbose=verbose, **kwargs)
 
 class Linear():
     '''
@@ -469,10 +480,15 @@ class NoWorkerTypeInteraction():
     Generate BLM constraints so that for a fixed firm type, worker types effects must all be the same.
 
     Arguments:
+        nnt (int or list of ints or None): time periods to constrain; None is equivalent to range(nt)
         nt (int): number of time periods
     '''
 
-    def __init__(self, nt=2):
+    def __init__(self, nnt=None, nt=2):
+        if nnt is None:
+            self.nnt = range(nt)
+        else:
+            self.nnt = to_list(nnt)
         self.nt = nt
 
     def _get_constraints(self, nl, nk):
@@ -486,10 +502,10 @@ class NoWorkerTypeInteraction():
         Returns:
             (dict of NumPy Arrays): {'G': None, 'h': None, 'A': A, 'b': b}, where G, h, A, and b are defined in the quadratic programming model
         '''
-        nt = self.nt
-        A = np.zeros(shape=(nt * (nl - 1) * nk, nt * nl * nk))
-        for period in range(nt):
-            row_shift = period * (nl - 1) * nk
+        nnt, nt = self.nnt, self.nt
+        A = np.zeros(shape=(len(nnt) * (nl - 1) * nk, nt * nl * nk))
+        for i, period in enumerate(nnt):
+            row_shift = i * (nl - 1) * nk
             col_shift = period * nl * nk
             for k in range(nk):
                 for l in range(nl - 1):
@@ -645,10 +661,15 @@ class StationaryFirmTypeVariation():
     Generate BLM constraints so that the firm type induced variation of worker-firm pair effects is the same in all periods. In particular, this is equivalent to setting A2 = (np.mean(A2, axis=1) + A1.T - np.mean(A1, axis=1)).T.
 
     Arguments:
+        nnt (int or list of ints or None): time periods to constrain; None is equivalent to range(1, nt)
         nt (int): number of time periods
     '''
 
-    def __init__(self, nt=2):
+    def __init__(self, nnt=None, nt=2):
+        if nnt is None:
+            self.nnt = range(1, nt)
+        else:
+            self.nnt = to_list(nnt)
         self.nt = nt
 
     def _get_constraints(self, nl, nk):
@@ -662,18 +683,22 @@ class StationaryFirmTypeVariation():
         Returns:
             (dict of NumPy Arrays): {'G': None, 'h': None, 'A': A, 'b': b}, where G, h, A, and b are defined in the quadratic programming model
         '''
-        nt = self.nt
-        A = np.zeros(shape=((nt - 1) * nl * nk, nt * nl * nk))
-        for period in range(nt - 1):
-            row_shift = period * nl * nk
+        nnt, nt = self.nnt, self.nt
+        A = np.zeros(shape=(len(nnt) * nl * nk, nt * nl * nk))
+        for i, period in enumerate(nnt):
+            row_shift = i * nl * nk
             col_shift = period * nl * nk
             for l in range(nl):
                 for k1 in range(nk):
                     for k2 in range(nk):
-                        A[row_shift + k1, col_shift + k2] = -(1 / nk)
-                        A[row_shift + k1, col_shift + nl * nk + k2] = (1 / nk)
-                    A[row_shift + k1, col_shift + k1] += 1
-                    A[row_shift + k1, col_shift + nl * nk + k1] -= 1
+                        # Baseline is period 1
+                        A[row_shift + k1, (col_shift - period * nl * nk) + k2] = -(1 / nk)
+                        # Comparison is period > 1
+                        A[row_shift + k1, col_shift + k2] = (1 / nk)
+                    # Baseline is period 1
+                    A[row_shift + k1, (col_shift - period * nl * nk) + k1] += 1
+                    # Comparison is period > 1
+                    A[row_shift + k1, col_shift + k1] -= 1
                 row_shift += nk
                 col_shift += nk
 
