@@ -3,27 +3,27 @@ Defines class CREEstimator, which uses a trace approximation to estimate the CRE
 '''
 # import logging
 # from pathlib import Path
-import pyamg
+from tqdm.auto import tqdm, trange
+import time, json
 import numpy as np
 import pandas as pd
-from bipartitepandas.util import ParamsDict, logger_init
 from scipy.sparse import csc_matrix, coo_matrix, diags, linalg, eye
-import time
+from pyamg import ruge_stuben_solver as rss
+from paramsdict import ParamsDict
+from bipartitepandas.util import logger_init
 # import pyreadr
 # import os
 # from multiprocessing import Pool, TimeoutError
 # from timeit import default_timer as timer
 # import argparse
-import json
 # import itertools
-from tqdm.auto import tqdm, trange
 
 # NOTE: multiprocessing isn't compatible with lambda functions
 def _gteq1(a):
     return a >= 1
 
 # Define default parameter dictionary
-_cre_params_default = ParamsDict({
+cre_params = ParamsDict({
     'ncore': (1, 'type_constrained', (int, _gteq1),
         '''
             (default=1) Number of cores to use.
@@ -50,21 +50,6 @@ _cre_params_default = ParamsDict({
     #     ''', '>= 1')
 })
 
-def cre_params(update_dict=None):
-    '''
-    Dictionary of default cre_params. Run tw.cre_params().describe_all() for descriptions of all valid parameters.
-
-    Arguments:
-        update_dict (dict or None): user parameter values; None is equivalent to {}
-
-    Returns:
-        (ParamsDict) dictionary of cre_params
-    '''
-    new_dict = _cre_params_default.copy()
-    if update_dict is not None:
-        new_dict.update(update_dict)
-    return new_dict
-
 def _pipe_qcov(df, e1, e2):
     v1 = df.eval(e1)
     v2 = df.eval(e2)
@@ -92,35 +77,34 @@ def _pd_to_np(df, colr, colc, colv, nr, nc):
 class CREEstimator:
     '''
     Uses multigrid and partialing out to solve two way Fixed Effect model.
+
+    Arguments:
+        data (Pandas DataFrame): cross-section labor data. Data contains the following columns:
+
+            i (worker id)
+
+            j1 (firm id 1)
+
+            j2 (firm id 2)
+
+            y1 (compensation 1)
+
+            y2 (compensation 2)
+
+            t1 (last period of observation 1)
+
+            t2 (last period of observation 2)
+
+            w1 (weight 1)
+
+            w2 (weight 2)
+
+            m (0 if stayer, 1 if mover)
+
+            cs (0 if not in cross section, 1 if in cross section)
+        cre_params (ParamsDict or None): dictionary of parameters for CRE estimation. Run tw.cre_params().describe_all() for descriptions of all valid parameters. None is equivalent to tw.cre_params().
     '''
     def __init__(self, data, params=None):
-        '''
-        Arguments:
-            data (Pandas DataFrame): cross-section labor data. Data contains the following columns:
-
-                i (worker id)
-
-                j1 (firm id 1)
-
-                j2 (firm id 2)
-
-                y1 (compensation 1)
-
-                y2 (compensation 2)
-
-                t1 (last period of observation 1)
-
-                t2 (last period of observation 2)
-
-                w1 (weight 1)
-
-                w2 (weight 2)
-
-                m (0 if stayer, 1 if mover)
-
-                cs (0 if not in cross section, 1 if in cross section)
-            cre_params (ParamsDict or None): dictionary of parameters for CRE estimation. Run tw.cre_params().describe_all() for descriptions of all valid parameters. None is equivalent to tw.cre_params().
-        '''
         # Start logger
         logger_init(self)
         # self.logger.info('initializing CREEstimator object')
@@ -548,7 +532,7 @@ class CREEstimator:
 
         self.logger.info('preparing linear solver')
         M = 1 / self.within_params['var_psi'] * eye(self.nf) + 1 / self.within_params['var_eps'] * Jd.transpose() * Jd
-        self.ml = pyamg.ruge_stuben_solver(M)
+        self.ml = rss(M)
 
     def __compute_posterior_var(self, rng=None):
         '''
